#ifndef FLTK_MT_Fl_MC_Widget_H
#define FLTK_MT_Fl_MC_Widget_H

// Copyright (C) 1999-2002, Matevz Tadel.
// This file is part of fltk-mt. License: GNU LGPL version 2.

#include <FL/Fl_Widget.H>
#include <FL/Fl.H>

#include <list>

// MouseEVentTrigger structure
struct Fl_MEVT {
  // Data that is shared among several instances of Fl_MC_Widget<>
  // there are additional arguments to the callback function;
  //   one of them is widget specific void* Fl_MC_Widget<>::mMC_data;

  // Events that will do the callback;
  // ME_drag:	  only the first drag is sent (think menu)
  // ME_alldrags: all drags will be passed; the final release even if the cursur is not inside
  // the widget. 
  enum MEvent_e { ME_push=1, ME_release=2, ME_drag=4, ME_alldrags=8 };
  enum MMtype_e { MM_all, MM_some, MM_any }; // ModifierMask_type
  typedef void (callback_f)(Fl_Widget*, MEvent_e, void*, int);

  int		events; 	// MEvents or-ed
  int		button;		// mouse button ... as in FL_RIGHT_MOUSE
  int		modifier_mask;	// as for Fl::event_state()
  MMtype_e	modifier_mask_type;
  callback_f*	mc_cb_foo;
  int		mc_type;
  Fl_Color	active_color;

  Fl_MEVT(int e, int b, int mm=0, MMtype_e mmt=MM_any,
	  callback_f* foo=0, int ut=0, Fl_Color ac=FL_BLACK) :
    events(e), button(b), modifier_mask(mm), modifier_mask_type(mmt),
    mc_cb_foo(foo), mc_type(ut), active_color(ac) {}
  void callback(callback_f* foo, int ut=0) {
    mc_cb_foo = foo; mc_type = ut;
  }
};

typedef list<Fl_MEVT>			lFl_MEVT_t;
typedef list<Fl_MEVT>::iterator		lFl_MEVT_i;
typedef list<Fl_MEVT>::const_iterator		lFl_MEVT_ci;

/**************************************************************************/

template<class T_Widget=Fl_Widget>
class Fl_MC_Widget : public T_Widget {
  // Wraps a Fl widget and handles mouse events
  //   Triggers are defined via a pointer to a list of Fl_MEVT structures.
  //   Static list is provided as a default option, but method
  //   void defMC_trigs() must be called to set it.
private:
  // Stoopid drag handlers ... ev_buttons go bannana after FL_LEAVE
  lFl_MEVT_ci	hCurTrig;	// current trigger ... drag/release
  bool		hDragNotified;
  bool		hOtherHandler;	// push was not handled here ... wait for a release
protected:
  const lFl_MEVT_t* 	mMC_trigs;
  void*	 	      	mMC_data;
public:
  Fl_MC_Widget(int x, int y, int w, int h, const char* t=0) :
    T_Widget(x, y, w, h, t) {
    mMC_trigs=0; mMC_data=0;
    hCurTrig = 0; hDragNotified = hOtherHandler = false;
  }
  virtual int handle(int ev);

  void setMC_trigs(const lFl_MEVT_t* t) { mMC_trigs=t; }
  void defMC_trigs() { mMC_trigs=&sMC_trigs; }
  void setMC_data(void* d) { mMC_data=d; }
  void setMCs(const lFl_MEVT_t* t, void* d) { mMC_trigs=t; mMC_data=d; }

  const lFl_MEVT_t* getMC_trigs() const { return mMC_trigs; }
  void*    	    getMC_data()  const { return mMC_data; }

  static lFl_MEVT_t sMC_trigs;
};

template<class T_Widget> lFl_MEVT_t Fl_MC_Widget<T_Widget>::sMC_trigs;

template<class T_Widget> int
Fl_MC_Widget<T_Widget>::handle(int ev) {
  bool handled = false;
  if(!hOtherHandler && mMC_trigs && (ev==FL_PUSH || ev==FL_DRAG || ev==FL_RELEASE)) {
    if(hCurTrig!=0) {
      handled = true; // don't let anybody mess with MY events
      bool inside = Fl::event_inside(this);
      if(ev==FL_DRAG) {
	bool dragp = hCurTrig->events & (Fl_MEVT::ME_drag | Fl_MEVT::ME_alldrags);
	if(hCurTrig->active_color && (hCurTrig->events & Fl_MEVT::ME_alldrags)==0) {
	  if(!inside && labelcolor()) { labelcolor(0); redraw(); }
	  if(inside && !labelcolor()) { labelcolor(hCurTrig->active_color); redraw(); }
	}
	if(!hDragNotified || (hCurTrig->events & Fl_MEVT::ME_alldrags)) {
	  if(hCurTrig->mc_cb_foo && dragp)
	    (hCurTrig->mc_cb_foo)(this, Fl_MEVT::ME_drag, mMC_data, hCurTrig->mc_type);
	  hDragNotified = true;
	}
      } else if(ev==FL_RELEASE) {
	if(Fl::event_button()==hCurTrig->button) {
	  if(hCurTrig->mc_cb_foo && (hCurTrig->events & Fl_MEVT::ME_release) &&
	     (inside || (hCurTrig->events & Fl_MEVT::ME_alldrags)))
	    {
	      (hCurTrig->mc_cb_foo)(this, Fl_MEVT::ME_release, mMC_data, hCurTrig->mc_type);
	    }
	}
	// Don't count on any other release ... reset state
	if(hCurTrig->active_color && labelcolor()) { labelcolor(0); redraw(); }
	hCurTrig = 0; hDragNotified = false;
      }
    } else if(ev==FL_PUSH) { // if no current ... all i want is a push
      for(lFl_MEVT_ci i=mMC_trigs->begin(); i!=mMC_trigs->end(); ++i) {
	if(Fl::event_button()==i->button) {
	  switch(i->modifier_mask_type) {
	  case Fl_MEVT::MM_all:
	    if(((i->modifier_mask & Fl::event_state()) == i->modifier_mask) ||
	       i->modifier_mask==0)
	      handled = true;
	    break;
	  case Fl_MEVT::MM_some:
	    if(((i->modifier_mask & Fl::event_state()) != 0) ||
	       i->modifier_mask==0)
	      handled = true;
	    break;
	  case Fl_MEVT::MM_any:
	    handled = true;
	    break;
	  } // end switch(i->modifier_mask_type)
	  if(handled) {
	    if(i->active_color) { labelcolor(i->active_color); redraw(); }
	    hCurTrig = i;
	    if(i->mc_cb_foo && (hCurTrig->events & Fl_MEVT::ME_push))
	      (i->mc_cb_foo)(this, Fl_MEVT::ME_push, mMC_data, i->mc_type);
	    break;
	  }
	}
      }
      if(!handled) hOtherHandler = true;
    }
  }
  if(hOtherHandler && ev==FL_RELEASE) hOtherHandler = false;
  return handled ? 1 : T_Widget::handle(ev);
}

#endif
