// The color chooser object and the color chooser popup.  The popup
// is just a window containing a single color chooser and some boxes
// to indicate the current and cancelled color.

#ifndef Fl_RGBA_Chooser_H
#define Fl_RGBA_Chooser_H

#include <FL/Fl_Group.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Value_Input.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_SWM.H>
#include <FL/Fl_RGBA_Button.H>

class Flcc_HueBox_A : public Fl_Widget {
  int handle(int);
  void draw();
  int px, py;
public:
  Flcc_HueBox_A(int X, int Y, int W, int H) : Fl_Widget(X,Y,W,H), px(0), py(0) {}
};

class Flcc_ValueBox_A : public Fl_Widget {
  int handle(int);
  void draw();
  int py;
public:
  Flcc_ValueBox_A(int X, int Y, int W, int H) : Fl_Widget(X,Y,W,H), py(0) {}
};

class Flcc_AlphaBox_A : public Fl_Widget {
  int handle(int);
  void draw();
  int py;
public:
  Flcc_AlphaBox_A(int X, int Y, int W, int H) : Fl_Widget(X,Y,W,H), py(0) {}
};

class Flcc_Value_Input_A : public Fl_Value_Input {
public:
  int format(char*);
  Flcc_Value_Input_A(int X, int Y, int W, int H) : Fl_Value_Input(X,Y,W,H) {}
};

/**************************************************************************/

class Fl_RGBA_Chooser : public Fl_Window, public Fl_SWM_Client {
  Fl_RGBA_Button     *master;

  Flcc_HueBox_A      huebox;
  Flcc_ValueBox_A    valuebox;
  Flcc_AlphaBox_A    alphabox;
  Fl_Choice	     choice;
  Flcc_Value_Input_A rvalue;
  Flcc_Value_Input_A gvalue;
  Flcc_Value_Input_A bvalue;
  Flcc_Value_Input_A avalue;
  Fl_Button	     ok_butt;
  Fl_RGBA_Button     ok_color;
  Fl_RGBA_Button     cc_color;
  Fl_Button	     cc_butt;
  Fl_Box	     resize_box;

  float hue_, saturation_, value_;
  float r_, g_, b_, a_;
  void set_valuators();
  static void rgb_cb(Fl_Widget*, void*);
  static void mode_cb(Fl_Widget*, void*);
  static void ok_cb(Fl_Button* b, Fl_RGBA_Chooser* c);
  static void set_cb(Fl_Button* b, Fl_RGBA_Chooser* c);
  static void undo_cb(Fl_Button* b, Fl_RGBA_Chooser* c);
  static void xit_cb(Fl_Button* b, Fl_RGBA_Chooser* c);
public:
  void set_master();
  void undo();
  void xit();
  void set_ok_butt();

  int mode() {return choice.value();}
  float hue() const {return hue_;}
  float saturation() const {return saturation_;}
  float value() const {return value_;}
  float r() const {return r_;}
  float g() const {return g_;}
  float b() const {return b_;}
  float a() const {return a_;}
  int hsv(float,float,float,float a=-1.0);
  int rgb(float,float,float,float a=-1.0);
  static void hsv2rgb(float, float, float,float&,float&,float&);
  static void rgb2hsv(float, float, float,float&,float&,float&);

  virtual void TraceManager(Fl_Widget* w);

  Fl_RGBA_Chooser(Fl_RGBA_Button* w, const char* l=0);
};

#endif
