// Copyright (C) 1999-2002, Matevz Tadel.
// This file is part of fltk-mt. License: GNU LGPL version 2.

#ifndef Fl_SWM_H
#define Fl_SWM_H

#include <FL/Fl_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Value_Input.H>

#include <list>

typedef list<Fl_Window*>		lpFl_Window_t;
typedef list<Fl_Window*>::iterator	lpFl_Window_i;

class Fl_Scaled_Windows_Manager;

struct SWM_Size_Range {
  int wl, hl, wh, hh; // low/high width/height
  int dwf, dhf;	      // delta w/h factor
  SWM_Size_Range(int a, int b, int c, int d, int e=1, int f=1) :
    wl(a), hl(b), wh(c), hh(d), dwf(e), dhf(f) {}
};

/**************************************************************************/
// Fl_SWM_Client
/**************************************************************************/

class Fl_SWM_Client {
  // Interface to provide a Fl_Window derived class with Fl_SWM info
protected:
  Fl_Scaled_Windows_Manager* swm_manager;
  Fl_Window*		     swm_window;
  SWM_Size_Range*	     swm_size_range;
public:
  Fl_SWM_Client() : swm_manager(0), swm_window(0), swm_size_range(0) {}
  virtual ~Fl_SWM_Client();

  virtual Fl_Scaled_Windows_Manager* GetManager() const { return swm_manager; }
  virtual void TraceManager(Fl_Widget* w);

  void Show();
  void HotSpot();

  void SetShowCB(Fl_Button* b)    { b->callback((Fl_Callback*) s_Show, this); }
  void SetHotSpotCB(Fl_Button* b) { b->callback((Fl_Callback*) s_HotSpot, this); }

  static void s_Show(Fl_Button* b, Fl_SWM_Client* c)    { c->Show(); }
  static void s_HotSpot(Fl_Button* b, Fl_SWM_Client* c) { c->HotSpot(); }

  friend class Fl_Scaled_Windows_Manager;
};

/**************************************************************************/
// Fl_Scaled_Windows_Manager
/**************************************************************************/

class Fl_Scaled_Windows_Manager {
  // Stores size information and maintains own window for size control
  // and a list (map?) of windows under its control

  // Size state

  int			old_width, old_height, old_fs;
  int			ctr_width, ctr_height, ctr_fs, ctr_w, ctr_h;
  int			bu_width, bu_height, bu_fs, bu_w, bu_h;

  bool			in_rescale;

  static int calc_width(Fl_Group* g, int w, int fs);
protected:
  // Size control window and its widgets

  Fl_Window*		wCtrl;
  Fl_Value_Input*	wWSkip; // if non 0 hardcodes the width (else width 'X')
  Fl_Value_Input*	wHSkip; // added to fontsize
  Fl_Value_Input*	wFontSize;

  // Adopted windows

  lpFl_Window_t		mWindows;

  // The work horse

public:
  static void ResizeGroup(Fl_Group* g, int ow, int oh, int nw, int nh, int nfs);

  // Methods

   Fl_Scaled_Windows_Manager(int fs=14, int h=6, int w=0);
  ~Fl_Scaled_Windows_Manager();

  void AdoptWindow(Fl_Window* w);
  void PrepareGroup(Fl_Group* g);
  void ResizeWindow(Fl_Window* w, int ow, int oh, int nw, int nh, int nfs);
  void RemoveWindow(Fl_Window* w);

  void KillAllWindows();

  int  cell_w() const { return in_rescale ? ctr_width : old_width; }
  int  cell_h() const { return in_rescale ? ctr_height :old_height; }
  int  cell_fontsize() const { return in_rescale ? ctr_fs : old_fs; }
  bool in_rescale_p() const { return in_rescale; }

  void Try();
  void Go();
  void Restore();
  static void s_Try(Fl_Widget* w, Fl_Scaled_Windows_Manager* x) { x->Try(); }
  static void s_Restore(Fl_Widget* w, Fl_Scaled_Windows_Manager* x) { x->Restore(); }
  static void s_Go(Fl_Widget* w, Fl_Scaled_Windows_Manager* x) { x->Go(); }

  void Show()	 { wCtrl->show(); }
  void HotSpot() { wCtrl->hotspot(wCtrl->w()/2, wCtrl->h()/2); Show(); }

  static Fl_Scaled_Windows_Manager* SearchManager(Fl_Widget* w);
};

#endif
