#include <FL/Fl_RGBA_Button.H>
#include <FL/Fl_RGBA_Chooser.H>
#include <FL/Fl.H>
#include <FL/fl_draw.h>

Fl_RGBA_Button::Fl_RGBA_Button(int x, int y, int w, int h, char const *t) 
  : Fl_Button(x, y, w, h, t), rgba_chooser(0),
    on_change_cb(0), on_change_ud(0)
{
  r = g = b = 0.5; a = 1; 
  callback((Fl_Callback*)default_cb, this);
}

Fl_RGBA_Button::~Fl_RGBA_Button() {
  delete rgba_chooser;
}

/**************************************************************************/

void Fl_RGBA_Button::set_rgba(float R, float G, float B, float A) {
  r = R; g = G; b = B; a = A;
  redraw();
}

void Fl_RGBA_Button::set_rgba(Fl_RGBA_Button* w) {
  r = w->r; g = w->g; b = w->b; a = w->a;
  redraw();
}

void Fl_RGBA_Button::spawn_chooser() {
  if(!rgba_chooser) {
    rgba_chooser = new Fl_RGBA_Chooser(this, label());
    rgba_chooser->TraceManager(this);
  }
  rgba_chooser->hotspot(rgba_chooser);
  rgba_chooser->show();
}

void Fl_RGBA_Button::wipe_chooser() {
  rgba_chooser = 0;
}

void Fl_RGBA_Button::default_cb(Fl_RGBA_Button* w, Fl_RGBA_Button* ud){
  if(w==ud) {
    w->spawn_chooser();
  } else {
    if(w->on_change()) (w->on_change())(w, w->on_change_data());
    w->redraw();
  }
}
/**************************************************************************/

static int th;

void Fl_RGBA_Button::draw() {
  draw_box();
  if (a==1) {
    fl_rectf(x()+Fl::box_dx(box()),
	     y()+Fl::box_dy(box()),
	     w()-Fl::box_dw(box()),
	     h()-Fl::box_dh(box()), (uchar)(255*r), (uchar)(255*g), (uchar)(255*b));
  } else {
    th = h()-Fl::box_dh(box());
    fl_draw_image(generate_achip, this, 
		  x()+Fl::box_dx(box()),
		  y()+Fl::box_dy(box()),
		  w()-Fl::box_dw(box()), th);
  }
  draw_label();
}

void Fl_RGBA_Button::generate_achip(void* vv, int X, int Y, int W, uchar* buf) {
  Fl_RGBA_Button *v = (Fl_RGBA_Button*)vv;
  if(th==0) return;
  float a = v->a, ia = 1.0f-a;
  float vr = v->r, vg = v->g, vb = v->b;
  uchar ro = (uchar)(vr*255.0f);
  uchar go = (uchar)(vg*255.0f);
  uchar bo = (uchar)(vb*255.0f);
  uchar rg = (uchar)(255*(a*vr + ia*0.8f));
  uchar gg = (uchar)(255*(a*vg + ia*0.4f));
  uchar bg = (uchar)(255*(a*vb + ia*0.4f));
  uchar r, g, b;
  if ((Y&0x07)==4) {
    r = rg; g = gg; b = bg;
  } else {
    r = (uchar)(255*(a*vr + ia*0.8f));
    g = (uchar)(255*(a*vg + ia*0.8f));
    b = (uchar)(255*(a*vb + ia*0.8f));
  }
  int xa = (int)((float)W/(float)th*(float)Y);
  for (int x = X; x < X+W; x++) {
    if (x>xa) {
      if ((x&0x07)==4) {
	*buf++ = rg; *buf++ = gg; *buf++ = bg;
      } else {
	*buf++ = r; *buf++ = g; *buf++ = b;
      }
    } else {
      *buf++ = ro; *buf++ = go; *buf++ = bo;
    }
  }
}

