// Copyright (C) 1999-2002, Matevz Tadel.
// This file is part of fltk-mt. License: GNU LGPL version 2.

#include <FL/Fl_Widget_Labeler.H>
#include <FL/Fl_Widget.H>

/************************************************************************/
// protected
/************************************************************************/

void
Fl_Widget_Labeler::new_chunk() {
  _chunks.push_back( new char[_chunk_size] );
  _cur_chunk_pos = _chunks.back();
  _cur_chunk_end = _cur_chunk_pos + _chunk_size;
}

/************************************************************************/
// public
/************************************************************************/

Fl_Widget_Labeler::Fl_Widget_Labeler(int s) : _chunk_size(s) {
  _cur_chunk_end = _cur_chunk_pos = 0;
}

Fl_Widget_Labeler::~Fl_Widget_Labeler() {
  for(list<char*>::iterator i=_chunks.begin(); i!=_chunks.end(); ++i)
    delete [] *i;
}

void
Fl_Widget_Labeler::label_widget(Fl_Widget* w, const char* l) throw(string) {
  int  strl  = strlen(l) + 1;
  if(strl > _chunk_size)
    throw(string("Fl_Widget_Labeler::label_widget chunk size too small"));
  if(_cur_chunk_end - _cur_chunk_pos < strl) new_chunk();
  strcpy(_cur_chunk_pos, l);
  w->label(_cur_chunk_pos);
  _cur_chunk_pos += strl;
}

void
Fl_Widget_Labeler::labelf_widget(Fl_Widget* w, const char* fmt, ...) throw(string) {
  static char* buf	= 0;
  static int   buf_size = 0;
  if(buf_size != _chunk_size) {
    delete [] buf;
    buf = new char[_chunk_size];
    buf_size = _chunk_size;
  }
  va_list ap;
  va_start(ap,fmt);
  int n = vsnprintf(buf, _chunk_size, fmt, ap);
  va_end(ap);
  if(n >= _chunk_size)
    throw(string("Fl_Widget_Labeler::labelf_widget chunk size too small"));
  if(n >= _cur_chunk_end - _cur_chunk_pos)
    new_chunk();
  strcpy(_cur_chunk_pos, buf);
  w->label(_cur_chunk_pos);
  _cur_chunk_pos += n+1;
}
